% Interactive interface for super-nested arrays
%
% Written by Chun-Lin Liu
% E-mail : cl.liu@caltech.edu
% Project website: http://systems.caltech.edu/dsp/students/clliu/SuperNested.html
%
% Our Papers:
% Journal
% [1] C.-L. Liu and P. P. Vaidyanathan, “Super Nested Arrays: Linear Sparse Arrays with Reduced Mutual Coupling - Part I: Fundamentals,” IEEE Transactions on Signal Processing, vol. 64, no. 15, pp. 3997-4012, Aug. 2016. 
% [2] C.-L. Liu and P. P. Vaidyanathan, “Super Nested Arrays: Linear Sparse Arrays with Reduced Mutual Coupling - Part II: High-Order Extensions,” IEEE Transactions on Signal Processing, vol. 64, no. 16, pp. 4203-4217, Aug. 2016. 
% Conference
% [3] C.-L. Liu and P. P. Vaidyanathan, “Super Nested Arrays: Sparse Arrays with Less Mutual Coupling than Nested Arrays,” in Proc. of 2016 IEEE International Conference on Acoustics Speech and Signal Processing (ICASSP 2016), pp. 2976-2980, Shanghai, China, Mar. 2016.
% [4] C.-L. Liu and P. P. Vaidyanathan, “High Order Super Nested Arrays,” in Proc. of the Ninth IEEE Sensor Array and Multichannel Signal Processing Workshop (SAM 2016), Rio de Janeiro, Brazil, Jul. 2016.
% 
% Last revised on August 14, 2016
%
%
% Usage:	Run this program directly
%			Check (V) on the 2D representation as sensors
%
% Push Buttons:
% 1. New 2D Representations: Open a new 2D Representation with given N1
% and N2
% 2. Show Array Profile: Display 1D, 2D Representations along with the
% weight function w(m)
% 3. Load Super Nested arrays: Load Super-Nested arrays for some Q
% 4. Clear interface: Remove all sensors




function interactive_interface( N1, N2 )


if ( nargin == 0 )
	% Default N1 and N2
    clear; clc; close all;
    N1 = 5; N2 = 5;
end



button_width = 200;
button_height = 25;
width = 12;
margin = 8;

% Identify size
Xmax = max( width * (N1+1), button_width*2 + margin ) + 2 * margin;
Ymax = margin * 4 + button_height*2 + width * N2;

f = figure('Position', [300, 300, Xmax, Ymax]);
set(f, 'Name', ['N1 = ', num2str(N1), ', N2 = ', num2str(N2)]) 

% Sensor locations
sensor_chkbox = zeros(N2, N1+1);
for nn2 = 1 : N2
    for nn1 = 1 : N1+1
        sensor_chkbox(nn2, nn1) = uicontrol( f, ...
                    'Style', 'checkbox',...
                    'Value', (nn2 == 1 || nn1 == N1+1), ...
                    'Position', [width*(nn1-1)+margin, width*(nn2-1)+button_height*2+margin*3, width, width]);
    end
end

% Display sensor locations
create_new_btn = uicontrol( f, ...
                        'Style', 'pushbutton', ...
                        'String', 'New 2D Representations', ...
                        'Position', [margin, margin*2+button_height, button_width, button_height], ...
                        'Callback', @create_new_btn_Callback);
    function create_new_btn_Callback(hObject, eventdata, handles)
        % Prompt user to enter N1 and N2
        while(1)
            dialog_answer = inputdlg({'Enter N1:','Enter N2:'}, ...
                                    'Please specify N1 and N2', ...
                                    1, ...
                                    {'75', '11'});
            if (length(dialog_answer) == 2)
                N1_entered = str2double( dialog_answer{1} );
                N2_entered = str2double( dialog_answer{2} );

                % Check N1 and N2. They need to be positive integers
                if ( N1_entered == floor(N1_entered) && N1_entered >= 1 && N2_entered == floor(N2_entered) && N2_entered >= 1)
                    break;
                else
                    uiwait( msgbox('Invalid Value! N1 and N2 have to be positive integers', 'Error','error') )
                end
            else
                return;
            end
        end
        
        interactive_interface( N1_entered, N2_entered );
    end

% display weight function
show_coarray_btn = uicontrol( f, ...
                        'Style', 'pushbutton', ...
                        'String', 'Show Array Profile', ...
                        'Position', [margin, margin, button_width, button_height], ...
                        'Callback', @show_coarray_btn_Callback);
    function show_coarray_btn_Callback(hObject, eventdata, handles)
        c = zeros(size(sensor_chkbox));
        for n2 = 1 : N2
            for n1 = 1 : N1+1
                c(n2,n1) = get( sensor_chkbox(n2,n1), 'Value');
            end
        end
        c_vec = find( reshape(c.', [], 1) ).';
        
        
        if ( isempty( c_vec ) )
            uiwait( msgbox('No sensors', 'Error','error') )
            return;
        end
        
        disp(['Number of sensors: ', num2str(length(c_vec))])
        disp(['Sensor location: ', num2str(c_vec)]);
        
        % Visualize on a figure;
        ff = figure;
        set(ff, 'Name', ['Array profile: N1 = ', num2str(N1), ', N2 = ', num2str(N2)])
        % 1-D representation
        subplot(2, 2, 1)
        plot(c_vec, zeros(size(c_vec)), '.');
        grid on;
        xlim([0, max(c_vec)+1]);
        xlabel('$n$', 'interpreter', 'latex')
        title('1D Representation', 'interpreter', 'latex')
        
        % 2-D representation
        subplot(2, 2, 2)
        plot(mod(c_vec-1, N1+1) + 1, floor((c_vec-1)/(N1+1)) + 1, '.');
        grid on;
        axis([0, N1+2, 0, N2+1])
        title('2D Representation', 'interpreter', 'latex')
        
        
        [V, w] = weight_function(c_vec, 'V');
        subplot(2, 2, 3)
        plot(V( V >= 0 ), w( V >= 0 ), 'o-')
        xlabel('Coarray location $m$', 'interpreter', 'latex');
        ylabel('$w(m)$', 'interpreter', 'latex');
        grid on;
        disp([  'w(1) = ', num2str( w(V==1) ), ...
              ', w(2) = ', num2str( w(V==2) ), ...
              ', w(3) = ', num2str( w(V==3) ), ...
              ', w(4) = ', num2str( w(V==4) ), ...
              ', w(5) = ', num2str( w(V==5) ), ...
              ', w(6) = ', num2str( w(V==6) ), ...
              ', w(7) = ', num2str( w(V==7) ), ...
              ', w(8) = ', num2str( w(V==8) )])
        title(['Number of sensors: ', num2str(length(c_vec)), ', Total aperture = ', num2str( num2str(V(end)) )], 'interpreter', 'latex');
        xlim([0, N2*(N1+1)-1])
        
        subplot(2, 2, 4)
        plot(V( V >= 0 ), w( V >= 0 ) > 0, 'o-')
        xlabel('Coarray location $m$', 'interpreter', 'latex');
        ylabel('$\mathbf{1}( w(m) > 0 )$', 'interpreter', 'latex');
        grid on;
        axis([0, N2*(N1+1)-1, 0, 2])
        
        if ( sum(w == 0) == 0 )
            title('Restricted array', 'interpreter', 'latex');
        else
            title('General array', 'interpreter', 'latex');
        end
        
        
        disp(['Number of holes: ', num2str(sum(w == 0))])
        disp(['Hole locations: ', num2str(V(w == 0).')])
    end

% Load some known configurations
load_super_nested_btn = uicontrol( f, ...
                        'Style', 'pushbutton', ...
                        'String', 'Load Super Nested Arrays', ...
                        'Position', [margin*2+button_width, margin*2+button_height, button_width, button_height], ...
                        'Callback', @load_super_nested_btn_Callback);
    function load_super_nested_btn_Callback(hObject, eventdata, handles)
        while (1)
            dialog_answer = inputdlg('Enter Q:', 'Please specify Q', 1, {'2'});
            if (length(dialog_answer) == 1)
                Q = str2double(dialog_answer{1});
                if ( Q == floor(Q) && Q >= 1)
                    break;
                else
                    uiwait( msgbox('Invalid Value! Q has to be a positive integer', 'Error','error') )
                end
            else
                return;
            end
        end
        S = super_nested(N1, N2, Q);
        for jj = 1 : N2
            for ii = 1 : N1+1
                set( sensor_chkbox(jj,ii), 'Value', sum(S == (jj-1)*(N1+1)+ii));
            end
        end
    end

clear_interface_btn = uicontrol( f, ...
                        'Style', 'pushbutton', ...
                        'String', 'Clear Sensors', ...
                        'Position', [margin*2+button_width, margin, button_width, button_height], ...
                        'Callback', @clear_interface_btn_Callback);
    function clear_interface_btn_Callback(hObject, eventdata, handles)
        for jj = 1 : N2
            for ii = 1 : N1+1
                set( sensor_chkbox(jj,ii), 'Value', 0);
            end
        end
    end

end
